@@ -29,6 +29,7 @@ typedef struct buffy_t
   short notified;		/* user has been notified */
   short magic;			/* mailbox type */
   short newly_created;		/* mbox or mmdf just popped into existence */
+  time_t last_visited;		/* time of last exit from this mailbox */
 }
 BUFFY;
 
@@ -43,3 +44,6 @@ void mutt_update_mailbox (BUFFY * b);
 /* fixes up atime + mtime after mbox/mmdf mailbox was modified
    according to stat() info taken before a modification */
 void mutt_buffy_cleanup (const char *buf, struct stat *st);
+
+/* mark mailbox just left as already notified */
+void mutt_buffy_setnotified (const char *path);
