@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
- * Copyright (C) 1998-2000 Thomas Roessler <roessler@does-not-exist.org>
+ * Copyright (C) 1998-2001,2007 Thomas Roessler <roessler@does-not-exist.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -43,11 +43,10 @@
 #include <limits.h>
 
 #ifndef _POSIX_PATH_MAX
-#include <posix1_lim.h>
+#include <limits.h>
 #endif
 
 #include "dotlock.h"
-#include "config.h"
 
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
@@ -554,7 +553,7 @@ dotlock_deference_symlink (char *d, size_t l, const char *path)
       char linkpath[_POSIX_PATH_MAX];
       int len;
 
-      if ((len = readlink (pathptr, linkfile, sizeof (linkfile))) == -1)
+      if ((len = readlink (pathptr, linkfile, sizeof (linkfile) - 1)) == -1)
       {
 	/* perror (pathptr); */
 	return -1;
