@@ -121,6 +121,8 @@ static void make_from (ENVELOPE *hdr, char *buf, size_t len, int do_lists)
     snprintf (buf, len, "To %s", mutt_get_name (hdr->to));
   else if (me && hdr->cc)
     snprintf (buf, len, "Cc %s", mutt_get_name (hdr->cc));
+  else if (me && hdr->bcc)
+    snprintf (buf, len, "Bcc %s", mutt_get_name (hdr->bcc));
   else if (hdr->from)
     strfcpy (buf, mutt_get_name (hdr->from), len);
   else
