@@ -39,7 +39,7 @@ imap_auth_res_t imap_auth_login (IMAP_DATA* idata, const char* method)
     return IMAP_AUTH_UNAVAIL;
   }
 
-  if (mutt_account_getlogin (&idata->conn->account))
+  if (mutt_account_getuser (&idata->conn->account))
     return IMAP_AUTH_FAILURE;
   if (mutt_account_getpass (&idata->conn->account))
     return IMAP_AUTH_FAILURE;
